//
//  The MENU CLASS
//
// isDisplayable() - return boolean whether this menu should be displayed.
// getTitle() - return the title or label for this menu.
// getImage() - return the image for this menu.
// getItems() - returns the array of MenuItems or Menus which this menu contains.
//
// $Id: class_Menu.js,v 1.6 2004/01/27 17:03:48 burgner Exp $
function Menu(title,raw,notype)
{
    this.title = title;
    this.raw = raw;
    this.img = "";
    this.type = "";
    this.items = new Array();
    this.parse(this.raw, notype);
}

Menu.prototype.isDisplayable = function()
{
    if(this.title == "Alerts")
    {
        if(this.items.length == 0)
            return false;
    }
    return true;
}

Menu.prototype.getTitle = function()
{
    // on IE, the Toolbar Settings has no text, but
    // I can't find how to do that here...
    if(this.title == "Toolbar Settings")
        return "";
    if(this.title == "Bearbeiten")
        return "";
    else if(this.title == "Alerts")
        return "";
    else
        return this.title;
}

Menu.prototype.getImage = function()
{
    return this.img;
}

Menu.prototype.getItems = function()
{
    return this.items;
}

Menu.prototype.parse = function(txt, notype)
{
    var end;
    if(notype != null)
    {
        //        alert("Not trying to find menu type");
        end = 0;
    }
    else
    {
        end = txt.indexOf('\x18');
        this.type = txt.substr(0, end);
    }
    var menuLineItems = txt.substr(end);
    //var testEol = txt.indexOf('\x19');
    //var mlItems2 = txt.substr(testEol);
    //alert("Type is '" + this.type + "' and menuLineItems: " + menuLineItems +'\n'
          //+ ' EOL: ' + testEol + '\n'
          //+ ' mlItems2: ' + mlItems2
          //);
    this.parseLineItems(menuLineItems);
    this.determineImage();
}

Menu.prototype.determineImage = function()
{
    // most times images for menus are empty.
    // however, some get an image.
    if(this.type == '05c.bmp')
        this.img = "chrome://yahootoolbar/content/icons/bm.gif";
    else
    {
        var tmp = this.title;
        if(tmp == "Toolbar Settings")
            this.img = "chrome://yahootoolbar/content/icons/setti.gif";
        if(tmp == "Bearbeiten")
            this.img = "chrome://yahootoolbar/content/icons/setti.gif";
        else if(tmp == "Alerts")
            this.img = "chrome://yahootoolbar/content/icons/bell.gif";
    }
}

Menu.prototype.parseLineItems = function(txt)
{
    // find the first ^V, that's the start of the menu
    // then ^V to ^Y^W is one item in the menu
    this.offset = txt.indexOf('\x16', this.offset);
    // ^A = \x01
    // ^B = \x02
    // ^C = \x03
    // ^V = \x16 (MO)
    // ^W = \x17 (MC)
    // ^X = \x18 (NV)
    // ^Y = \x19 (EOL)

    // this algorithm works if there are no nested menus.
    // a simple menu looks like:
    // ^V .... ^W
    // ^V .... ^W
    //
    // but the problem is that a menu line item with a nested menu looks like:
    // ^V <title> ^X ^C ^Y ^V <subtitle> ^X ... <link>^Y ^W
    // ...
    // ^Y^V <subtitle> ^X <link> ^Y ^W
    // ^W
    // so, here, we need to determine if string represents a simple line item
    // or if its a nested menu.
    do {
        var startMenuItem = txt.indexOf('\x16', this.offset);
        // fetch the title... up to the first ^X
        var endTitle = txt.indexOf('\x18', startMenuItem);
        var title = txt.substr((startMenuItem + 1), (endTitle - (startMenuItem + 1)));
        
        // now we need to examine the character after the ^X
        // if it's a ^C, we have a nested menu,
        var idChar = txt.charAt((endTitle + 1));
        if(idChar == '\x03')
        {
            var nestedMenuString = null;
            nestedMenuString = this.findNestedMenu(txt, (endTitle+3));
            this.items.push(new Menu(title, nestedMenuString, 1));
            this.offset += 1 + title.length + 3 + nestedMenuString.length + 1;
        } else {
            endMenuItem = txt.indexOf('\x17', startMenuItem);
            if(endMenuItem == -1)
            {
                // this always happens in a nested menu... but it's a good
                // sanity check anyway.
                //alert("HEY! couldn't find a ^Y to end this!");
                break;
            }
            var menuLine = txt.substr((startMenuItem+1), (endMenuItem - (startMenuItem+1)));
            this.items.push(new MenuItem(menuLine)); 
            this.offset += ((endMenuItem + 1) - startMenuItem);
        }
    } while(this.offset < txt.length);
    //alert("Done parsing menu named " + this.title);
}

Menu.prototype.findNestedMenu = function(txt, start, alreadyFound)
{
    var retString = "";
    var endNestedMenu = start;
    var vCount = 1;
    var a = "";

    while (vCount != 0) {
      a = txt.charAt(endNestedMenu);
      if (a == '\x16') {
        vCount++;
      }
      if (a == '\x17') {
        vCount--;
      }
      endNestedMenu++;
    }

    return txt.substr(start, ((endNestedMenu - start) - 1));
}
Menu.prototype.dumpItems = function(indent)
{
    var msg = "Menu title: " + this.title + "\n";
    var len = this.items.length;
    var i=0;
    for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(indent != null)
            msg += indent;
        if(mli.constructor == MenuItem)
        {
            
            msg += "Txt: " + mli.menuText + " Link: " + mli.menuLink + "\n";
        }
        else
        {
            // this is a nested menu!
            msg += "NESTED: \n";
            var newindent = (indent != null)? indent+"\t" : "\t";
            msg += mli.dumpItems(newindent);
            if(indent != null) msg += indent;
            msg+= "END NESTED\n";
        }
    }
    return msg;
}
Menu.prototype.getTxt = function()
{
	var retVal;
    var len = this.items.length;
    var i=0;
    
	for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(mli.constructor == MenuItem)
        {
            retVal = mli.menuText;
        }
    }
    return retVal;
}
Menu.prototype.htmlDumpItems = function(indent)
{
    var msg = "<br><b>'" + this.title + "'</b> \n";
    var len = this.items.length;
    var i=0;
    for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(indent != null)
            msg += indent;

        if(mli.constructor == MenuItem)
        {
            if (mli.menuText != "-") {
                msg += "<br>Txt: '" + mli.menuText + "' Link: '" + mli.menuLink + "'\n";
            }
        }
        else
        {
            // this is a nested menu!
            msg += "<div style=\"margin-left: 40px;\"> \n";
            var newindent = (indent != null) ? indent + "\t" : "\t";
            msg += mli.dumpItems(newindent);
            
            if (indent != null) 
                msg += indent;
            msg+= "</div> \n";
        }
    }
    return msg;
}
Menu.prototype.breakLine = function(line)
{
      var tmp1 = "";
      var tmp2 = "";
      var start = 30;
      while (start < line.length) {
        tmp1 = line.substring(0, start);
        tmp2 = line.substring(start, line.length);
        line = tmp1 + "\n" + tmp2;
        start += 30;
      }
      return line;
}

